#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""Create a dataset label from attributes."""
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud="otc")
dataset_id = "heZw7Oh7Ha0eiFIzkm8"
attrs = {
    "labels": [
        {
            "name": "Cat",
            "property": {
                "@modelarts:color": "#3399ff",
                "@modelarts:default_shape": "bndbox",
            },
        }
    ]
}

dataset_label = conn.modelartsv2.create_dataset_label(dataset_id, **attrs)
print(dataset_label)
