# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import openstack

from otcextensions.tests.functional import base


class TestOtcextensionsInitialization(base.BaseFunctionalTest):

    def test_basic(self):
        conn = openstack.connect(
            vendor_hook='otcextensions.sdk:load'
        )

        self.assertTrue(hasattr(conn.dns, 'add_router_to_zone'))
        list(conn.volume_backup.backup_policies())
        list(conn.obs.containers())
