#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Create Auto-Scaling Group.
"""
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')

attrs = {
    "scaling_group_name": "TestGroup",
    "scaling_configuration_id": "0d138717-e79d-4fa7-b8aa-37ddec1b3ce7",
    "desire_instance_number": 0,
    "min_instance_number": 0,
    "max_instance_number": 10,
    "vpc_id": "26ca2783-dc40-4e3a-95b1-5a0756441e12",
    "available_zones": ["eu-de-01", "eu-de-03"],
    "networks": [
        {
            "id": "25d24fc8-d019-4a34-9fff-0a09fde6a9cb"
        }
    ]
}

group = conn.auto_scaling.create_group(**attrs)
print(group)
