#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Adding OBS Dump Task.
"""
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')

stream_name = 'test-stream'
attrs = {
    "destination_type": "OBS",
    "obs_destination_descriptor": {
        "task_name": "newtask",
        "consumer_strategy": "LATEST",
        "agency_name": "dis_admin_agency",
        "destination_file_type": "text",
        "obs_bucket_path": "obsbucket",
        "file_prefix": "",
        "partition_format": "yyyy/MM/dd/HH/mm",
        "record_delimiter": "|",
        "deliver_time_interval": 30
    }
}
result = conn.dis.create_dump_task(stream_name, **attrs)
print(result)
