#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
 Create virtual interface
"""
import openstack
from otcextensions import sdk

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')
sdk.register_otc_extensions(conn)

attrs = {
    "direct_connect_id": "direct_connect_uuid",
    "vgw_id": "virtual_gateway_uuid",
    "type": "public",
    "service_type": "vpc",
    "vlan": 2556,
    "bandwidth": 10,
    "local_gateway_v4_ip": "217.150.159.57/30",
    "remote_gateway_v4_ip": "217.150.159.58/30",
    "route_mode": "static",
    "remote_ep_group_id": "remote_ep_group_uuid"
}

vi = conn.dcaas.create_virtual_interface(**attrs)
print(vi)
