# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class Sms(resource.Resource):
    base_path = '/notifications/sms'

    # capabilities
    allow_create = True

    #: Message ID, which is unique
    message_id = resource.Body('message_id')
    #: Request ID, which is unique.
    request_id = resource.Body('request_id')
    #: Phone number. The phone number must be preceded by
    #:  a plus sign (+) and a country code.
    endpoint = resource.Body('endpoint')
    #: SMS message content.
    message = resource.Body('message')
