#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Modify VPC Channel Healthcheck
"""
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')
attrs = {
    "http_code": "200",
    "path": "/vpc/demo",
    "port": 22,
    "method": "GET",
    "protocol": "http",
    "threshold_abnormal": 5,
    "threshold_normal": 2,
    "time_interval": 10,
    "timeout": 5,
    "enable_client_ssl": False
}
updated = conn.apig.modify_vpc_channel_healthcheck(
    gateway="gateway_id",
    vpc_channel="vpc_channel_id",
    **attrs
)
